@@ -134,14 +134,14 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
             None,
         );
 
-        // Schedule title updates every 1s
+        // Schedule title updates every 2s
         let timer_id = TimerId::new(Topic::UpdateTitles, 0);
         if !self.scheduler.scheduled(timer_id) {
             self.scheduler.schedule(
                 EventPayload::new(RioEventType::Rio(RioEvent::UpdateTitles), unsafe {
                     rio_window::window::WindowId::dummy()
                 }),
-                Duration::from_secs(1),
+                Duration::from_secs(2),
                 true,
                 timer_id,
             );
@@ -236,36 +236,28 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
                         let context_manager = route.window.screen.ctx_mut();
                         let grid = context_manager.current_grid_mut();
 
-                        // Check for damage and trigger appropriate damage events
-                        let mut has_damage = false;
+                        let mut request_pending_redraw = false;
+
                         for (_key, grid_context) in grid.contexts().iter() {
-                            if let Some(mut terminal) =
+                            let ctx = grid_context.context();
+
+                            // In this case we know we have to render something that's pending.
+                            if ctx.renderable_content.pending_update.is_dirty() {
+                                request_pending_redraw = true;
+                            }
+
+                            if let Some(terminal) =
                                 grid_context.context().terminal.try_lock_unfair()
                             {
-                                if terminal.is_fully_damaged() {
-                                    has_damage = true;
-                                    break;
-                                } else {
-                                    // Check for partial damage
-                                    if let rio_backend::crosswords::TermDamage::Partial(
-                                        _,
-                                    ) = terminal.damage()
-                                    {
-                                        has_damage = true;
-                                        break;
-                                    }
-                                }
+                                terminal.emit_damage_event();
                             }
                         }
 
-                        // If we found damage, emit a TerminalDamaged event
-                        if has_damage {
-                            self.event_proxy.send_event(
-                                RioEventType::Rio(RioEvent::TerminalDamaged {
-                                    route_id,
-                                    damage: rio_backend::event::TerminalDamage::Full, // Simplified for now
-                                }),
-                                window_id,
+                        if request_pending_redraw {
+                            route.schedule_redraw(
+                                &mut self.scheduler,
+                                &self.event_proxy,
+                                route.window.screen.ctx().current_route(),
                             );
                         }
                     }
@@ -303,31 +295,50 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
                                 // Skip rendering if no actual damage
                                 return;
                             }
-
-                            // For cursor-only damage, we can be more selective
-                            if matches!(
-                                damage,
-                                rio_backend::event::TerminalDamage::CursorOnly
-                            ) {
-                                // Only render cursor changes if cursor is visible or blinking
-                                let cursor_state = terminal.cursor();
-                                if !cursor_state.is_visible() {
-                                    return;
-                                }
-                            }
-                            drop(terminal); // Release lock before requesting redraw
+                            drop(terminal);
 
                             // Clear the one-time render flag if it was set
                             if route.window.needs_render_after_occlusion {
                                 route.window.needs_render_after_occlusion = false;
                             }
 
-                            // Request immediate redraw for responsive UI
-                            route.request_redraw();
+                            let ctx = route.window.screen.ctx_mut().current_mut();
+                            ctx.renderable_content
+                                .pending_update
+                                .invalidate(damage, &ctx.terminal);
+                            route.schedule_redraw(
+                                &mut self.scheduler,
+                                &self.event_proxy,
+                                route_id,
+                            );
                         }
                     }
                 }
             }
+            RioEventType::Rio(RioEvent::UpdateGraphics { route_id, queues }) => {
+                if let Some(route) = self.router.routes.get_mut(&window_id) {
+                    // Check if this is the current route
+                    if route_id == route.window.screen.ctx().current_route() {
+                        // Process graphics directly in sugarloaf
+                        let sugarloaf = &mut route.window.screen.sugarloaf;
+
+                        for graphic_data in queues.pending {
+                            sugarloaf.graphics.insert(graphic_data);
+                        }
+
+                        for graphic_data in queues.remove_queue {
+                            sugarloaf.graphics.remove(&graphic_data);
+                        }
+
+                        // Request a redraw to display the updated graphics
+                        route.schedule_redraw(
+                            &mut self.scheduler,
+                            &self.event_proxy,
+                            route_id,
+                        );
+                    }
+                }
+            }
             RioEventType::Rio(RioEvent::PrepareUpdateConfig) => {
                 let timer_id = TimerId::new(Topic::UpdateConfig, 0);
                 let event = EventPayload::new(
@@ -1299,6 +1310,22 @@ impl ApplicationHandler<EventPayload> for Application<'_> {
                 // println!("Time elapsed in render() is: {:?}", duration);
                 // }
 
+                if route
+                    .window
+                    .screen
+                    .ctx()
+                    .current()
+                    .renderable_content
+                    .pending_update
+                    .is_dirty()
+                {
+                    route.schedule_redraw(
+                        &mut self.scheduler,
+                        &self.event_proxy,
+                        route.window.screen.ctx().current_route(),
+                    );
+                }
+
                 if self.config.renderer.strategy.is_game() {
                     route.request_redraw();
                 }
